==> audiochain-1.1.5-src.tgz: org/audiochain/io/DelayedBlockAudioDataReader.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.io;

import static java.lang.System.*;
import static java.util.Arrays.*;

import java.io.IOException;

abstract public class DelayedBlockAudioDataReader extends BlockAudioDataReader {

	protected final float frameRate;
	private FrameObserver frameObserver;
	private int delayInFrames;
	private int delayBufferSize;
	private int delayTimeInMilliseconds;
	private Delay delay;
	private int[] block;

	DelayedBlockAudioDataReader(AudioDataReader reader) {
		super(reader);
		this.frameRate = 0;
	}

	public DelayedBlockAudioDataReader(AudioDataReader reader, float frameRate) {
		super(reader);
		this.frameRate = frameRate;
	}

	public DelayedBlockAudioDataReader(AudioDataReader reader, float frameRate, FrameObserver frameObserver) {
		super(reader);
		this.frameRate = frameRate;
		this.frameObserver = frameObserver;
	}

	@Override
	public int read(int[] data) throws IOException {
		int len = super.read(data);
		if (len == -1 && !omit && block != null) {
			int blockSize = getBlockSize();
			for (int i = 0, n = delayBufferSize / blockSize; i < n; i++) {
				fill(block, 0);
				delay.delay(block, 0, blockSize);
				processDelayedBlock(block, 0, blockSize);
			}
			block = null;
		}
		return len;
	}
	
	@Override
	protected final void processBlock(int[] data, int offset, int length) {
		if (frameObserver != null) {
			setDelayInFrames((int)frameObserver.getLatencyInFrames());
		}
		if (block == null) {
			calcDelayBuffer();
		}
		arraycopy(data, offset, block, 0, length);
		delay.delay(block, 0, length);
		processDelayedBlock(block, 0, length);
	}

	private void calcDelayBuffer() {
		delayBufferSize = delayInFrames << 1;
		delay = new Delay(delayBufferSize);
		block = new int[getBlockSize()];
		delayTimeInMilliseconds = (int) (delayInFrames * 1000 / frameRate);
	}

	abstract protected void processDelayedBlock(int[] block, int offset, int length);

	public int getDelayInFrames() {
		return delayInFrames;
	}

	public void setDelayInFrames(int delayInFrames) {
		if (this.delayInFrames != delayInFrames) {
			this.delayInFrames = delayInFrames;
			calcDelayBuffer();
		}
	}

	@Override
	public void setBlockSizeInFrames(int blockSizeInFrames) {
		super.setBlockSizeInFrames(blockSizeInFrames);
		calcDelayBuffer();
	}

	public int getDelayTimeInMilliseconds() {
		return delayTimeInMilliseconds;
	}

	public void setDelayTimeInMilliseconds(int delayTimeInMilliseconds) {
		this.delayTimeInMilliseconds = delayTimeInMilliseconds;
		setDelayInFrames((int) (frameRate * delayTimeInMilliseconds / 1000));
	}
}

==> audiochain-1.1.6-src.tgz: org/audiochain/io/DelayedBlockAudioDataReader.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.io;

import static java.lang.System.arraycopy;
import static java.util.Arrays.fill;

import java.io.IOException;

abstract public class DelayedBlockAudioDataReader extends BlockAudioDataReader {

	protected final float frameRate;
	private FrameObserver frameObserver;
	private int delayInFrames;
	private int delayBufferSize;
	private int delayTimeInMilliseconds;
	private Delay delay;
	private int[] block;

	DelayedBlockAudioDataReader(AudioDataReader reader) {
		super(reader);
		this.frameRate = 0;
	}

	public DelayedBlockAudioDataReader(AudioDataReader reader, float frameRate) {
		super(reader);
		this.frameRate = frameRate;
	}

	public DelayedBlockAudioDataReader(AudioDataReader reader, float frameRate, FrameObserver frameObserver) {
		super(reader);
		this.frameRate = frameRate;
		this.frameObserver = frameObserver;
	}

	@Override
	public int read(int[] data) throws IOException {
		int len = super.read(data);
		if (len == -1 && !omit && block != null) {
			int blockSize = getBlockSize();
			for (int i = 0, n = delayBufferSize / blockSize; i < n; i++) {
				fill(block, 0);
				delay.delay(block, 0, blockSize);
				processDelayedBlock(block, 0, blockSize);
			}
			block = null;
		}
		return len;
	}
	
	@Override
	protected final void processBlock(int[] data, int offset, int length) {
		if (frameObserver != null) {
			setDelayInFrames((int)frameObserver.getLatencyInFrames());
		}
		if (block == null) {
			calcDelayBuffer();
		}
		arraycopy(data, offset, block, 0, length);
		delay.delay(block, 0, length);
		processDelayedBlock(block, 0, length);
	}

	private void calcDelayBuffer() {
		delayBufferSize = delayInFrames << 1;
		delay = new Delay(delayBufferSize);
		block = new int[getBlockSize()];
		delayTimeInMilliseconds = (int) (delayInFrames * 1000 / frameRate);
	}

	abstract protected void processDelayedBlock(int[] block, int offset, int length);

	public int getDelayInFrames() {
		return delayInFrames;
	}

	public void setDelayInFrames(int delayInFrames) {
		if (this.delayInFrames != delayInFrames) {
			this.delayInFrames = delayInFrames;
			calcDelayBuffer();
		}
	}

	@Override
	public void setBlockSizeInFrames(int blockSizeInFrames) {
		super.setBlockSizeInFrames(blockSizeInFrames);
		calcDelayBuffer();
	}

	public int getDelayTimeInMilliseconds() {
		return delayTimeInMilliseconds;
	}

	public void setDelayTimeInMilliseconds(int delayTimeInMilliseconds) {
		this.delayTimeInMilliseconds = delayTimeInMilliseconds;
		setDelayInFrames((int) (frameRate * delayTimeInMilliseconds / 1000));
	}
}
